## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
from CvPythonExtensions import *
# Needed for DomPyCity which inherits from PyCity
# (subclassed for additional functions)
import DomPyHelpers
import PyHelpers
import CvUtil
import ScreenInput
import CvScreenEnums

# Needed to check for non-numbers (specially search function)
import re

PyPlayer = DomPyHelpers.DomPyPlayer
PyCity = DomPyHelpers.DomPyCity

#	IMPORTANT INFORMATION
#	
#	All widget names MUST be unique when creating screens.  If you create
#	a widget named 'Hello', and then try to create another named 'Hello', it
#	will modify the first hello.
#
#	Also, when attaching widgets, 'Background' is a reserve word meant for
#	the background widget.  Do NOT use 'Background' to name any widget, but
#	when attaching to the background, please use the 'Background' keyword.

# globals
gc = CyGlobalContext()
ArtFileMgr = CyArtFileMgr()
localText = CyTranslator()

# Debugging help
def SpecDomPrint (stuff):
	stuff = "SpecDomAdv: " + stuff
	CvUtil.pyPrint (stuff)

#	Set up the base Coordinates here...
# Not sure if we still need this. This isn't used anywhere in the code.
#STANDARD_Z = -2.2
#TOP_Y = 96
#Y_SPACING = 32
#DATE_COLUMN = 70
#NAME_COLUMN = DATE_COLUMN + 90
#CULTURE_COLUMN = NAME_COLUMN + 130
#HAPPY_COLUMN = CULTURE_COLUMN + 90
#HEALTH_COLUMN = HAPPY_COLUMN + 100
#FOOD_COLUMN = HEALTH_COLUMN + 40
#PRODUCTION_COLUMN = FOOD_COLUMN + 40
#GOLD_COLUMN = PRODUCTION_COLUMN + 40
#GREATPEOPLE_COLUMN = GOLD_COLUMN + 40
#PRODUCING_COLUMN = GREATPEOPLE_COLUMN + 170

# What are we sorting by?
# Not sure if this is necessary as it doesn't seem to be used.
#iSortingBy = DATE
#iSortingDown = True

# Listbox identifiers...
# Not sure if this is necessary as it doesn't seem to be used.
#UNIT_CHOSEN = 0
#BUILDING_CHOSEN = 1
#PROJECT_CHOSEN = 2
#PROCESS_CHOSEN = 3

# Class CvSpecialDomesticAdvisor
# Class
# Class

class CvSpecialDomesticAdvisor:
	"Special Domestic Advisor Screen"

	# Called when cIV is first booted up and everytime you switch out of cIV
	def __init__(self):
		SpecDomPrint ('%s' % help (CyPlayer))
		self.NO_CITY = -1

		self.nScreenWidth = 1024
		self.nScreenLength = 768

		self.nTableX = 12
		self.nTableY = 42
		self.nTableWidth = 1000
		self.nTableLength = 400

		self.nCultureLevelX = 670
		self.nCultureLevelY = 450
		self.nCultureLevelDistance = 15
		self.nCultureLevelTextOffset = 110

		self.nGPLevelX = 870
		self.nGPLevelY = 450
		self.nGPLevelDistance = 30

		self.nFirstSpecialistX = 30
		self.nSpecialistY = 450
		self.nSpecialistWidth = 32
		self.nSpecialistLength = 32
		self.nSpecialistDistance = 60

		self.nPlusOffsetX = -4
		self.nMinusOffsetX = 16
		self.nPlusOffsetY = self.nMinusOffsetY = 30
		self.nPlusWidth = self.nPlusHeight = self.nMinusWidth = self.nMinusHeight = 20

		self.nSpecTextOffsetX = 0
		self.nSpecTextOffsetY = 50

		self.SCREEN_NAME = "DomesticAdvisor"
		self.EXIT_NAME = "DomesticExit"
		self.VIEW_NAME = "DomesticView"
		self.REDRAW_NAME = "DomesticRedraw"
		self.SPECIALIST_BUTTON_NAME = "SpecialistDomesticAdvisorButton"
		self.DEFAULT_AREA_NAME = "DefaultListBackground"
		self.SPECIAL_AREA_NAME = "SpecialListBackground"
		self.BACKGROUND_ID = "DomesticAdvisorBG"

		self.CULTURE_TEXT_NAME = "DomCultureText"
		self.GP_TEXT_NAME = "DomGPText"
		self.NUMBER_TEXT = "NUM"

		self.SPECIALIST_IMAGE_NAME = "DomCitizenImage"
		self.SPECIALIST_PLUS_NAME = "DomIncreaseSpecialist"
		self.SPECIALIST_MINUS_NAME = "DomDecreaseSpecialist"
		self.SPECIALIST_TEXT_NAME = "DomSpecialistText"

		self.m_nChosenCity = self.NO_CITY
#		self.m_nLastSpecialistChange = 0
#		self.m_bSpecialistChange = False
#		self.m_bSpecialistChangeType = False
		self.m_szMode = self.DEFAULT_AREA_NAME
		self.m_bHandlingSpecialistChange = False
		self.m_nTimesChanged = 0
		self.m_szOldSpecialists = ""

		# Default Sorting information (also, column numbering)
		self.DEF_COLUMN_DICT = {
			"DATE" : 0,
			"NAME" : 1,
			"LANDMARKS" : 2,
			"POPULATION" : 3,
			"GARRISON" : 4,
			"HAPPY" : 5,
			"HEALTH" : 6,
			"GROWTH" : 7,
			"FOOD" : 8,
			"PRODUCTION" : 9,
			"MAINTENANCE" : 10,
			"TRADE" : 11,
			"COMMERCE" : 12,
			"GOLD" : 13,
			"RESEARCH" : 14,
			"CULTURE_RATE" : 15,
			"CULTURE" : 16,
			"GREATPEOPLE_RATE" : 17,
			"GREATPEOPLE" : 18,
			"PRODUCING" : 19,
			}

		# Default Column Size Information (The total of all should add up to 1000)
		self.DEF_COLUMN_SIZE_DICT = {
			"DATE" : 80,
			"NAME" : 100,
			"LANDMARKS" : 80,
			"POPULATION" : 35,
			"GARRISON" : 30,
			"HAPPY" : 30,
			"HEALTH" : 30,
			"GROWTH" : 35,
			"FOOD" : 35,
			"PRODUCTION" : 40,
			"MAINTENANCE" : 30,
			"TRADE" : 30,
			"COMMERCE" : 40,
			"GOLD" : 40,
			"RESEARCH" : 40,
			"CULTURE_RATE" : 40,
			"CULTURE" : 55,
			"GREATPEOPLE_RATE" : 40,
			"GREATPEOPLE" : 45,
			"PRODUCING" : 145
			}

		# Default Sorting information (also, column numbering)
		# NOTE: NAME MUST be Column #1 in order to allow a selection to give the
		# Production management screen at the bottom.
		self.SPEC_COLUMN_DICT = {
			"RELIGIONS" : 0,
			"NAME" : 1,
			"SPECIALISTS" : 2,
			"HAPPY" : 3,
			"GROWTH" : 4,
			"FOOD" : 5,
			"PRODUCTION" : 6,
			"GOLD" : 7,
			"RESEARCH" : 8,
			"CULTURE_RATE" : 9,
			"CULTURE" : 10,
			"GREATPEOPLE_RATE" : 11,
			"GREATPEOPLE" : 12,
			"PRODUCING" : 13,
			}

		# Default Column Size Information (The total of all should add up to 1000)
		self.SPEC_COLUMN_SIZE_DICT = {
			"RELIGIONS" : 130,
			"NAME" : 100,
			"SPECIALISTS" : 225,
			"HAPPY" : 30,
			"GROWTH" : 35,
			"FOOD" : 35,
			"PRODUCTION" : 40,
			"GOLD" : 40,
			"RESEARCH" : 40,
			"CULTURE_RATE" : 40,
			"CULTURE" : 55,
			"GREATPEOPLE_RATE" : 40,
			"GREATPEOPLE" : 45,
			"PRODUCING" : 145,
			}

		# Values to check to see if we need to color the number
		self.PROBLEM_VALUES_DICT = {
			"GARRISON" : 1,
			"HAPPY" : 0,
			"HEALTH" : 0,
			"GROWTH" : 0,
			"FOOD" : 1,
			}

		# Values to change on an update
		# (True indicates update when we DON'T switch to/from food production)
		# Most of this MIGHT change because the computer might be switching
		# plots it's working or creating/removing specialists.

		# TODO: Unfortunately, right now, we have no way of knowing when it's
		# switching from food production to non-food production and vice versa.
		self.UPDATE_DICT = {
			"HAPPY": False,
			"GROWTH" : False,
			"FOOD": False,
			"PRODUCTION": True,
			"COMMERCE": False,
			"GOLD": False,
			"RESEARCH": False,
			"CULTURE_RATE": False,
			"GREATPEOPLE_RATE": False,
			"PRODUCING": True,
			"SPECIALISTS": False
			}

		self.HEADER_DICT = None
		self.JUSTIFY_DICT = None
		self.SPECIALIST_ICON_DICT = None
		self.CITY_PRODUCING_DICT = {}

		# Input handling functions
		self.DomesticScreenInputMap = {
			self.SPECIALIST_BUTTON_NAME		: self.SpecialistDomesticButton,
			self.SPECIALIST_PLUS_NAME : self.HandleSpecialist,
			self.SPECIALIST_MINUS_NAME : self.HandleSpecialist,
			self.EXIT_NAME			: self.DomesticExit,
			self.VIEW_NAME			: self.CityView,
			self.REDRAW_NAME		: self.Redraw,
			}

	def createDictionaries(self):

		if self.HEADER_DICT == None:
			# Header Information (Must be here, because C++ functions aren't
			# available upon startup of CIV)
			self.HEADER_DICT = {
				"DATE" : localText.getText("TXT_KEY_DOMESTIC_ADVISOR_FOUNDED", ()).upper(),
				"NAME" : localText.getText("TXT_KEY_DOMESTIC_ADVISOR_NAME", ()).upper(),
				"LANDMARKS" : localText.getText("TXT_KEY_WB_BUILDINGS", ()),
				"POPULATION" : localText.getText("TXT_KEY_POPULATION", ()).upper(),
				"GARRISON" : u"%c" % CyGame().getSymbolID(FontSymbols.STRENGTH_CHAR),
				"HAPPY" : u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR),
				"HEALTH" : u"%c" % CyGame().getSymbolID(FontSymbols.HEALTHY_CHAR),
#				"GROWTH" : u"%c" % gc.getBonusInfo(19).getChar(),
				"GROWTH" : u"%c" % CyGame().getSymbolID(FontSymbols.BAD_FOOD_CHAR),
				"FOOD" : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_FOOD).getChar()) ,
				"PRODUCTION" : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar()),
				"MAINTENANCE" : u"%c" % CyGame().getSymbolID(FontSymbols.BAD_GOLD_CHAR),
				"TRADE" : u"%c" % CyGame().getSymbolID(FontSymbols.TRADE_CHAR),
				"COMMERCE" : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_COMMERCE).getChar()),
				"GOLD" : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar()),
				"RESEARCH" : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar()),
				"CULTURE_RATE" : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_CULTURE).getChar()),
				"CULTURE" : localText.getText("TXT_KEY_TOTAL", ()),
				"GREATPEOPLE_RATE" : u"%c" % CyGame().getSymbolID(FontSymbols.GREAT_PEOPLE_CHAR),
				"GREATPEOPLE" : localText.getText("TXT_KEY_TOTAL", ()),
				"PRODUCING" : localText.getText("TXT_KEY_DOMESTIC_ADVISOR_PRODUCING", ()),
				"RELIGIONS" : localText.getText("TXT_KEY_ADVISOR_RELIGION", ()),
				"SPECIALISTS" : localText.getText("TXT_KEY_CONCEPT_SPECIALISTS", ())
			}

		if self.JUSTIFY_DICT == None:
			# Header Information (Must be here, because C++ functions aren't
			# available upon startup of CIV)
			self.JUSTIFY_DICT = {
				"DATE" : CvUtil.FONT_LEFT_JUSTIFY, 
				"NAME" : CvUtil.FONT_LEFT_JUSTIFY,
				"LANDMARKS" : CvUtil.FONT_LEFT_JUSTIFY,
				"POPULATION" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GARRISON" : CvUtil.FONT_RIGHT_JUSTIFY,
				"HAPPY" : CvUtil.FONT_RIGHT_JUSTIFY,
				"HEALTH" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GROWTH" : CvUtil.FONT_RIGHT_JUSTIFY,
				"FOOD" : CvUtil.FONT_RIGHT_JUSTIFY,
				"PRODUCTION" : CvUtil.FONT_RIGHT_JUSTIFY,
				"MAINTENANCE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"TRADE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"COMMERCE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GOLD" : CvUtil.FONT_RIGHT_JUSTIFY,
				"RESEARCH" : CvUtil.FONT_RIGHT_JUSTIFY,
				"CULTURE_RATE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"CULTURE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GREATPEOPLE_RATE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"GREATPEOPLE" : CvUtil.FONT_RIGHT_JUSTIFY,
				"PRODUCING" : CvUtil.FONT_LEFT_JUSTIFY,
				"RELIGIONS" : CvUtil.FONT_LEFT_JUSTIFY,
				"SPECIALISTS" : CvUtil.FONT_LEFT_JUSTIFY,
			}

		if self.SPECIALIST_ICON_DICT == None:
			# Header Information (Must be here, because C++ functions aren't
			# available upon startup of CIV)
			self.SPECIALIST_ICON_DICT = {
				0 : u"%c" % CyGame().getSymbolID(FontSymbols.BULLET_CHAR), # Citizen
				1 : u"%c" % CyGame().getSymbolID(FontSymbols.HAPPY_CHAR), # Priest
				2 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_CULTURE).getChar()), # Artist
				3 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_RESEARCH).getChar()), # Scientist
				4 : u"%c" %(gc.getCommerceInfo(CommerceTypes.COMMERCE_GOLD).getChar()), # Merchant
				5 : u"%c" %(gc.getYieldInfo(YieldTypes.YIELD_PRODUCTION).getChar())
				}

	def createPositions (self, screen):
		self.nScreenX = screen.centerX(0)
		self.nScreenY = screen.centerY(0)

	# Create a new screen, called DomesticAdvisor, using the file CvSpecialDomesticAdvisor.py for input
	def getScreen(self):
		return CyGInterfaceScreen(self.SCREEN_NAME, CvScreenEnums.DOMESTIC_ADVISOR)

	def getCurrentCity (self):
		if (self.m_nChosenCity != self.NO_CITY):
			iPlayer = PyPlayer(CyGame().getActivePlayer())
			cityList = iPlayer.getCityList()

			return PyCity( CyGame().getActivePlayer(), cityList[self.m_nChosenCity].getID() )
		else:
			return None

	def getNumSpecialistInfos (self):
		return len (self.SPECIALIST_ICON_DICT)

	# Screen construction function
	# This is the function that's called whenever F1 is pressed.
	def interfaceScreen(self):

		self.createDictionaries()

		# This will get the actual Player class into iPlayer
		iPlayer = PyPlayer(CyGame().getActivePlayer())

		screen = self.getScreen()
#		screen.setForcedRedraw (True)
		self.createPositions (screen)

		screen.setDimensions (self.nScreenX, self.nScreenY, self.nScreenWidth, self.nScreenLength)
		screen.showScreen(PopupStates.POPUPSTATE_IMMEDIATE, False)
	
		self.drawBasicScreen()

		self.drawScreen (self.m_szMode)

	def drawBasicScreen (self):

		screen = self.getScreen()

		# Here we set the background widget and exit button, and we show the screen
		screen.addPanel( self.BACKGROUND_ID, u"", u"", True, False, 0, 29, 1024, 562, PanelStyles.PANEL_STYLE_MAIN )
		#screen.addDDSGFC( self.BACKGROUND_ID, ArtFileMgr.getInterfaceArtInfo("SCREEN_BG").getPath(), 0, 29, 1024, 592, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		# Text Buttons
		screen.setText(self.EXIT_NAME, "Background", localText.getText("TXT_KEY_PEDIA_SCREEN_EXIT", ()).upper(), CvUtil.FONT_RIGHT_JUSTIFY, 1012, 564, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_CLOSE_SCREEN, -1, -1 )
		screen.setText(self.VIEW_NAME, "Background", "VIEW", CvUtil.FONT_RIGHT_JUSTIFY, 812, 564, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )
		screen.setText(self.REDRAW_NAME, "Background", "REDRAW", CvUtil.FONT_RIGHT_JUSTIFY, 612, 564, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		# Specialist Screen Toggle Button
		screen.addCheckBoxGFC(self.SPECIALIST_BUTTON_NAME , gc.getSpecialistInfo(0).getTexture(), ArtFileMgr.getInterfaceArtInfo("BUTTON_HILITE_SQUARE").getPath(), 12, 554, self.nSpecialistWidth, self.nSpecialistLength, WidgetTypes.WIDGET_CITIZEN, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )

		# Cultural Levels Text
		screen.setText (self.CULTURE_TEXT_NAME, "Background", self.HEADER_DICT["CULTURE_RATE"], CvUtil.FONT_LEFT_JUSTIFY, self.nCultureLevelX, self.nCultureLevelY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		nGameSpeed = CyGame().getGameSpeedType()
		iCount = 2
		for i in range (gc.getNumCultureLevelInfos()):
			pCultureLevel = gc.getCultureLevelInfo (i)
			nValue = pCultureLevel.getSpeedThreshold(nGameSpeed)
			if (nValue != 0):
				screen.setText (self.CULTURE_TEXT_NAME + str(i), "Background", "<font=2>" + pCultureLevel.getText() + "</font>", CvUtil.FONT_LEFT_JUSTIFY, self.nCultureLevelX, self.nCultureLevelY + (self.nCultureLevelDistance * iCount), -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				screen.setText (self.CULTURE_TEXT_NAME + self.NUMBER_TEXT + str(i), "Background", "<font=2>" + str(nValue) + "</font>", CvUtil.FONT_RIGHT_JUSTIFY, self.nCultureLevelX + self.nCultureLevelTextOffset, self.nCultureLevelY + (self.nCultureLevelDistance * iCount), -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				iCount += 1

		iPlayer = PyPlayer(CyGame().getActivePlayer())

		# GP Level Text
		screen.setText (self.GP_TEXT_NAME, "Background", self.HEADER_DICT["GREATPEOPLE_RATE"], CvUtil.FONT_RIGHT_JUSTIFY, self.nGPLevelX, self.nGPLevelY, -0.1, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
		screen.setText (self.GP_TEXT_NAME + self.NUMBER_TEXT, "Background", "<font=2>" + str (iPlayer.greatPeopleThreshold()) + "</font>", CvUtil.FONT_RIGHT_JUSTIFY, self.nGPLevelX, self.nGPLevelY + self.nGPLevelDistance, -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		# Header...
		#szText = "<font=4>" + localText.getText("TXT_KEY_DOMESTIC_ADVISOR_TITLE", ()).upper() + "</font>"
		#screen.setLabel( "DomesticTitleHeader", "Background", szText, CvUtil.FONT_CENTER_JUSTIFY, 472, 40, STANDARD_Z, FontTypes.TITLE_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		self.drawSpecialists()

	# Draw the specialist and their increase and decrease buttons
	def drawSpecialists (self):

		screen = self.getScreen()

		# Citizen Buttons
		for i in range( self.getNumSpecialistInfos() ):
		
			if (gc.getSpecialistInfo(i).isVisible()):
			
				szName = self.SPECIALIST_IMAGE_NAME + str(i)
				screen.setImageButton( szName, gc.getSpecialistInfo(i).getTexture(), self.nFirstSpecialistX + (self.nSpecialistDistance * i), self.nSpecialistY, self.nSpecialistWidth, self.nSpecialistLength, WidgetTypes.WIDGET_CITIZEN, i, -1 )
				screen.hide( szName )

		# Increase Specialists...
		for i in range( self.getNumSpecialistInfos() ):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_PLUS_NAME + str(i)
				screen.setButtonGFC( szName, u"", "", self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nPlusOffsetX, self.nSpecialistY + self.nPlusOffsetY, self.nPlusWidth, self.nPlusHeight, WidgetTypes.WIDGET_CHANGE_SPECIALIST, i, 1, ButtonStyles.BUTTON_STYLE_CITY_PLUS )
				screen.hide( szName )

		# Decrease specialists
		for i in range( self.getNumSpecialistInfos() ):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_MINUS_NAME + str(i)
				screen.setButtonGFC( szName, u"", "", self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nMinusOffsetX, self.nSpecialistY + self.nMinusOffsetY, self.nMinusWidth, self.nMinusHeight, WidgetTypes.WIDGET_CHANGE_SPECIALIST, i, -1, ButtonStyles.BUTTON_STYLE_CITY_MINUS )
				screen.hide( szName )

		# Specialist text
		for i in range (self.getNumSpecialistInfos()):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_TEXT_NAME + str(i)
				screen.setText (szName, "Background", "", CvUtil.FONT_LEFT_JUSTIFY, self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nSpecTextOffsetX, self.nSpecialistY + self.nSpecTextOffsetY, -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				screen.hide (szName)

	def showSpecialists (self):
		screen = self.getScreen()

		self.hideSpecialists()

		pCity = self.getCurrentCity()
		if (pCity):

			nPopulation = pCity.getPopulation()
			nFreeSpecial = pCity.totalFreeSpecialists()

			# For each specialist type
			for i in range( self.getNumSpecialistInfos() ):

				# Show all the specialist images
				szName = self.SPECIALIST_IMAGE_NAME + str(i)
				screen.show( szName )

				szName = self.SPECIALIST_TEXT_NAME + str(i)
				screen.setText (szName, "Background", str (pCity.getSpecialistCount(i)) + "/" + str (pCity.getMaxSpecialistCount(i)) + u" %c" % self.SPECIALIST_ICON_DICT[i], CvUtil.FONT_LEFT_JUSTIFY, self.nFirstSpecialistX + (self.nSpecialistDistance * i) + self.nSpecTextOffsetX, self.nSpecialistY + self.nSpecTextOffsetY, -0.1, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)
				screen.show (szName)

				# If the specialist is valid
				if (pCity.isSpecialistValid(i, 1) and (pCity.getForceSpecialistCount(i) < (nPopulation + nFreeSpecial))):
					szName = self.SPECIALIST_PLUS_NAME + str(i)
					screen.show( szName )

				if (pCity.getSpecialistCount(i) > 0 or pCity.getForceSpecialistCount(i) > 0 ):
					szName = self.SPECIALIST_MINUS_NAME + str(i)
					screen.show( szName )
							
	def hideSpecialists (self):
		screen = self.getScreen()

		# Hide Buttons
		for i in range( self.getNumSpecialistInfos() ):
			if (gc.getSpecialistInfo(i).isVisible()):
				szName = self.SPECIALIST_IMAGE_NAME + str(i)
				screen.hide (szName)
				szName = self.SPECIALIST_PLUS_NAME + str(i)
				screen.hide (szName)
				szName = self.SPECIALIST_MINUS_NAME + str(i)
				screen.hide (szName)
				szName = self.SPECIALIST_TEXT_NAME + str(i)
				screen.hide (szName)

	def hide (self, screen, screenType):
		screen.hide (screenType)
		if (screenType == self.SPECIAL_AREA_NAME):
			self.hideSpecialists()

	# Create the Default Domestic Advisor Screen
	def drawScreen (self, szMode):
		screen = self.getScreen()

		if (szMode == self.DEFAULT_AREA_NAME):
			self.unHighlight()
			self.hide (screen, self.SPECIAL_AREA_NAME)
		elif (szMode == self.SPECIAL_AREA_NAME):
			self.highlight()
			self.hide (screen, self.DEFAULT_AREA_NAME)
			self.showSpecialists ()
		else:
			return

		# Erase the flag?
		CyInterface().setDirty(InterfaceDirtyBits.MiscButtons_DIRTY_BIT, True)

		# Draw the city list...
		self.drawContents (szMode)

		screen.enableSort (szMode)

	# headers...
	def drawHeaders (self, szArea):

		# Get the screen
		screen = self.getScreen()

		if (szArea == self.DEFAULT_AREA_NAME):
			lKVPairs = self.DEF_COLUMN_DICT.items()
			dColSizeDict = self.DEF_COLUMN_SIZE_DICT
		elif (szArea == self.SPECIAL_AREA_NAME):
			lKVPairs = self.SPEC_COLUMN_DICT.items()
			dColSizeDict = self.SPEC_COLUMN_SIZE_DICT
		else:
			return

		# For each column
		for key, value in lKVPairs:
			# set the header
			screen.setTableColumnHeader (szArea, value, "<font=2>" + self.HEADER_DICT[key] + "</font>", dColSizeDict[key] )

	# Function to calculate value given a certain city and key
	def CalculateValue (self, pCity, szKey):
		# Initialize Return Value
		szReturn = ""

		if (szKey == "DATE"):
			# City founded date...
			szIDText = "Date" + str(pCity.getID())
			iTurnTime = pCity.getGameTurnFounded()
			szReturn = unicode(CyGameTextMgr().getTimeStr(iTurnTime, false))

		elif (szKey == "NAME"):
			# City name...
			szReturn = pCity.getName()

		elif (szKey == "LANDMARKS"):
			# Landmarks...

			# First look for Government Centers
			if pCity.isGovernmentCenter():
				szStar = u"%c" % CyGame().getSymbolID(FontSymbols.STAR_CHAR)
				szSilverStar = u"%c" % CyGame().getSymbolID(FontSymbols.SILVER_STAR_CHAR)
				# And distinguish between the Capital and the others (Forbidden Palace
				# and Versailles)
				if pCity.isCapital():
					szReturn += szStar
				else:
					szReturn += szSilverStar

			# add National Wonder Landmarks
			nWonder = 0
      # hard coded building id strings from xml (not sure if want to keep
			# hard-coded but can't think of an easier way of doing this)
			for nWonder in range (64,75):
				if pCity.hasBuilding(nWonder):
					# Use bullets as markers for National Wonders
					szBullet = u"%c" % CyGame().getSymbolID(FontSymbols.BULLET_CHAR)
					szReturn += szBullet
			# end National Wonder Landmarks

			if pCity.isPower():
				szPower = u"%c" % CyGame().getSymbolID(FontSymbols.POWER_CHAR)
				szReturn += szPower
			if pCity.isDisorder():
				szOccu = u"%c" % CyGame().getSymbolID(FontSymbols.OCCUPATION_CHAR)
				szAngry = u"%c" % CyGame().getSymbolID(FontSymbols.ANGRY_POP_CHAR)

				if pCity.isOccupation():
					szReturn += szOccu +":"+unicode(pCity.getOccupationTimer())
				else:
					szReturn += szAngry

		elif (szKey == "POPULATION"):
			# Population
			szReturn = unicode(pCity.getPopulation())

		elif (szKey == "GARRISON"):
			# Garrison
			szReturn = unicode(pCity.getMilitaryHappinessUnits())

		elif (szKey == "HAPPY"):
			# Replace Happiness/Unhappiness with Net Happiness...
			szReturn = unicode(pCity.getHappyPopulation() - pCity.getUnhappyPopulation())

		elif (szKey == "HEALTH"):
			# combine health/unhealthy... to Net City Health
			szReturn = unicode(pCity.getGoodHealth() - pCity.getBadHealth())

		elif (szKey == "GROWTH"):
			# Turns til Growth
 			nFood = pCity.foodDifference (True)

			# If this is a food production (i.e., worker or settler)
			if (pCity.isFoodProduction()):
				# We need to indicate there's no growth manually
				szReturn = "-"
			else:
				# Otherwise, just call FoodTurnsLeft
				szReturn = unicode (pCity.getFoodTurnsLeft())

			# Not enough food, so calculate how many turns until we starve.
			# We put this here because we still starve if building a food production
			if nFood < 0:
				# Use floor divide (//) because we want the number it'll drop below 0
				# (that's also the reason for the additional 1)
				szReturn = unicode ((pCity.getFood()+1) // nFood)

		elif (szKey == "FOOD"):
			# Replace Food/Food Used with Net Food AND Turns until Growth...

			# If this is a food production (i.e., worker or settler)
			if (pCity.isFoodProduction()):
				nFood = pCity.getCurrentProductionDifference (False, False) - pCity.getCurrentProductionDifference (True, False)
			else:
				nFood = pCity.foodDifference (True)
			szReturn = unicode(nFood)

		elif (szKey == "PRODUCTION"):
			# Production status
			szReturn = unicode(pCity.getCurrentProductionDifference (True, False))

		elif (szKey == "MAINTENANCE"):
			# City Maintanance...
			szReturn = unicode(pCity.getMaintenance())

		elif (szKey == "TRADE"):
			# Trade status...
			nTotalTradeProfit = 0

			# For each trade route possible
			for nTradeRoute in range (gc.getDefineINT("MAX_TRADE_ROUTES")):
				# Get the next trade city
				pTradeCity = pCity.getTradeCity(nTradeRoute)
				# Not quite sure what this does but it's in the MainInterface
				# and I pretty much C&Ped :p
				if (pTradeCity and pTradeCity.getOwner() >= 0):
					for j in range( YieldTypes.NUM_YIELD_TYPES ):
						iTradeProfit = pCity.calculateTradeYield(j, pCity.calculateTradeProfit(pTradeCity))

						# If the TradeProfit is greater than 0 and it to the total
						if ( iTradeProfit > 0 ):
							nTotalTradeProfit += iTradeProfit

			szReturn = unicode(nTotalTradeProfit)

		elif (szKey == "COMMERCE"):
			# Commerce status...
			szReturn = unicode(pCity.calculateGoldRate())

		elif (szKey == "GOLD"):
			# Gold status...
			szReturn = unicode(pCity.reallyGold())

		elif (szKey == "RESEARCH"):
			# Science rate...
			szReturn = unicode(pCity.getResearchRate())

		elif (szKey == "CULTURE_RATE"):
			# Culture status...
			szReturn = unicode(pCity.getCultureCommerce())

		elif (szKey == "CULTURE"):
			# Total Culture
			szReturn = unicode(pCity.getCulture())

		elif (szKey == "GREATPEOPLE_RATE"):
			# Great Person
			szReturn = unicode(pCity.getGreatPeopleRate())

		elif (szKey == "GREATPEOPLE"):
			# Great Person
			szReturn = unicode(pCity.getGreatPeopleProgress())

		elif (szKey == "PRODUCING"):
			# Producing

			# If there's something in the queue,
			if (pCity.getOrderQueueLength() > 0):

				# If it's not a process (i.e., Wealth, Research, Culture)
				if (not (pCity.isProductionProcess())):
					# It should have number of turns left attached to it
					# (Note: Put this in the front, otherwise, there's a chance it'll be
					# unable to be seen.)
					szReturn = "("
					szReturn += unicode(pCity.getProductionTurnsLeft())
					szReturn += ") "

				# Get the name of whatever it's producing.
				szReturn += pCity.getProductionName()

			# Otherwise we're not producing anything. Leave it blank.
			else:
				szReturn = "-"

			# Store what the city is producing
			self.CITY_PRODUCING_DICT[pCity.getName()] = szReturn

		elif (szKey == "RELIGIONS"):
			# Religions...

			lHolyCity = pCity.getHolyCity()
			if lHolyCity:
				iI = 0
				for iI in range(len(lHolyCity)):
					szReturn += u"%c" %(gc.getReligionInfo(lHolyCity[iI]).getHolyCityChar())
			lReligions = pCity.getReligions()
			if lReligions:
				iI = 0
				for iI in range(len(lReligions)):
					if lReligions[iI] in lHolyCity:
						continue
					szReturn += u"%c" %(gc.getReligionInfo(lReligions[iI]).getChar())

		elif (szKey == "SPECIALISTS"):
			# Specialists...

			# For each specialist type
			for i in range( self.getNumSpecialistInfos() ):

				# For each specialist
				for j in range( pCity.getSpecialistCount(i) ):
					szReturn += self.SPECIALIST_ICON_DICT[i]

		# return the final value
		return szReturn

	def drawDomesticButtons (self, bHighlight):
		screen = self.getScreen()
		screen.setState(self.SPECIALIST_BUTTON_NAME, bHighlight)		

	def highlight(self):
		self.m_szMode = self.SPECIAL_AREA_NAME
		self.drawDomesticButtons(True)
		
	def unHighlight(self):
		self.m_szMode = self.DEFAULT_AREA_NAME
		self.drawDomesticButtons(False)
		
	def ColorProblemCityValues (self, nValue, szKey):
		"Colors city values which we might want to alert the user to."

		if (self.PROBLEM_VALUES_DICT.has_key (szKey)):

			# Get the color to highlight with
			color = gc.getInfoTypeForString("COLOR_RED")

			# If we don't have a plain integer
			if (re.search ("[-+]", nValue)):
				# Color and return it
				return localText.changeTextColor (nValue, color)
			# If comparison is less than the good value
			# TODO (maybe): In the future we might want more than a less than number
			#               for now it's good though
			if (int(nValue) < self.PROBLEM_VALUES_DICT[szKey]):
				# Color and return it
				return localText.changeTextColor (nValue, color)

		# Otherwise, just return the regular value
		return nValue

	# Function to draw the contents of the cityList passed in
	def drawContents (self, szArea):

		# Get the screen and the player
		screen = self.getScreen()

		iPlayer = PyPlayer(CyGame().getActivePlayer())

		screen.moveToFront( "Background" )

		if (szArea == self.DEFAULT_AREA_NAME):
			dDict = self.DEF_COLUMN_DICT
		elif (szArea == self.SPECIAL_AREA_NAME):
			dDict = self.SPEC_COLUMN_DICT
		else:
			return

		# Build the table	
		screen.addTableControlGFC (szArea, len (dDict), self.nTableX, self.nTableY, self.nTableWidth, self.nTableLength, True, False, 32, 32, TableStyles.TABLE_STYLE_STANDARD )
		screen.enableSelect (szArea, True )

		# Loop through the cities
		cityList = iPlayer.getCityList()
		for i in range(len(cityList)):
			pLoopCity = PyCity( CyGame().getActivePlayer(), cityList[i].getID() )

			screen.appendTableRow (szArea)
#			screen.selectRow (szArea, i, True)

			# For each column
			for key, value in dDict.items():
				justify = self.JUSTIFY_DICT[key]
				if (justify == CvUtil.FONT_LEFT_JUSTIFY):
					funcTableWrite = screen.setTableText
				else:
					funcTableWrite = screen.setTableInt

				szValue = self.ColorProblemCityValues (self.CalculateValue (pLoopCity, key), key)
				funcTableWrite (szArea, value, i, szValue, "", WidgetTypes.WIDGET_GENERAL, -1, -1, justify)

		self.drawHeaders (szArea)

		if (self.m_nChosenCity < len(cityList) and \
				self.m_nChosenCity != self.NO_CITY):
			screen.selectRow (szArea, self.m_nChosenCity, True)
		else:
			self.m_nChosenCity = self.NO_CITY

		screen.moveToBack (self.BACKGROUND_ID)

		screen.updateAppropriateCitySelection (szArea, len( PyPlayer(CyGame().getActivePlayer()).getCityList() ) )

	def HandleSpecialist (self, inputClass):
#		SpecDomPrint ("Entered HandleSpecialist")

		pCity = self.getCurrentCity()
		if (pCity):

			self.m_bHandlingSpecialistChange = True
			self.m_szOldSpecialists = self.CalculateValue (pCity, "SPECIALISTS")
			self.m_nTimesChanged = 0
		return 0

	def SpecialistDomesticButton (self, inputClass):
		if (self.m_szMode == self.DEFAULT_AREA_NAME):
			# Switch to Specialist
			self.m_szMode = self.SPECIAL_AREA_NAME
		else:
			# Switch to Default
			self.m_szMode = self.DEFAULT_AREA_NAME

		self.drawScreen (self.m_szMode)

		return 1
		
	def DomesticExit (self, inputClass):
		self.m_bHandlingSpecialistChange
		return 0

	def CityView (self, inputClass):
		screen = self.getScreen()
		screen.hideScreen()	

		# Grab the city indicated by the index
		city = CyGlobalContext().getActivePlayer().getCity(self.m_nChosenCity)
		CyInterface().selectCity(city, True)
		# And jump straight to it
		CyEngine().setCityBillboardVisibility(True)
		return 1

	def Redraw (self, inputClass):
		# Redraw the screen
#		SpecDomPrint ("Redrawing screen")
		self.drawScreen (self.m_szMode)
		return 1

	# Will handle the input for this screen...
	def handleInput (self, inputClass):
		' Calls function mapped in DomesticAdvisorInputMap'
		# only get from the map if it has the key

#		SpecDomPrint ("Entered handleInput")
#		SpecDomPrint ("Func: %s, %d, %d" % (inputClass.getFunctionName(), inputClass.getData(), inputClass.getID()))

		# If we get that a listbox item was selected
		if ( inputClass.getNotifyCode() == NotifyCode.NOTIFY_LISTBOX_ITEM_SELECTED ):
			screen = self.getScreen()

			# Store the index
			nIndex = inputClass.getData()
			self.m_nChosenCity = nIndex

			if (self.m_szMode == self.SPECIAL_AREA_NAME):
				self.showSpecialists()

			# And pass it back to the screen
			screen.updateAppropriateCitySelection( self.m_szMode, len( PyPlayer(CyGame().getActivePlayer()).getCityList() ) )

			# TODO: Note, there is NO possible way we can tell which city is actually
			# being indicated. As such, we don't want to jump to a city which might
			# not be the correct one. Because of this we just pass it to the screen
			# who DOES have the information to determine which city it is.

			# Perhaps in the future this will be opened up to us so that we can make
			# the call from a higher level.

			# Because of this problem, I had to add a redraw function which sets the
			# list back to the known order so that the jump can't be correctly made.

##			city = gc.getActivePlayer().getCity(nIndex)
##			CyInterface().selectCity (city, True)

##			CyEngine().setCityBillboardVisibility(True)

		elif (inputClass.getNotifyCode() == NotifyCode.NOTIFY_CHARACTER):
			if (inputClass.getData() == int(InputTypes.KB_V)):
				return self.CityView (inputClass)
			elif (inputClass.getData() == int(InputTypes.KB_R)):
				return self.Redraw (inputClass)

		# Otherwise, we got something else
		elif (self.DomesticScreenInputMap.has_key(inputClass.getFunctionName())):
			'Calls function mapped in CvSpecDomesticAdvisor'
			# only get from the map if it has the key		

			# get bound function from map and call it
			return self.DomesticScreenInputMap.get(inputClass.getFunctionName())(inputClass)

		return 0
	
	def isFoodProduction (self, szProducing):
		return re.search ("Worker|Settler", szProducing)

	def isFoodProductionChange (self, pCity):
		szCity = pCity.getName()

		# If we already have noted what this city was producing
		if (self.CITY_PRODUCING_DICT.has_key (szCity)):
			# Get if we switched to/from a food production
			self.m_bFoodProductionSwitch = ((pCity.isFoodProduction () and \
																			 not (self.isFoodProduction (self.CITY_PRODUCING_DICT[szCity]))) or \
																			(not (pCity.isFoodProduction ()) and \
																			 self.isFoodProduction (self.CITY_PRODUCING_DICT[szCity])))
		else:
			# Otherwise, there's no way it's a food production switch
			self.m_bFoodProductionSwitch = False

	def updateScreen(self):

		self.drawScreen (self.m_szMode)

		screen = self.getScreen()
		iPlayer = PyPlayer(CyGame().getActivePlayer())

		if (self.m_szMode == self.DEFAULT_AREA_NAME):
			dDict = self.DEF_COLUMN_DICT
		elif (self.m_szMode == self.SPECIAL_AREA_NAME):
			dDict = self.SPEC_COLUMN_DICT
		else:
			return

		cityList = iPlayer.getCityList()
		# For each city
		for i in range(len(cityList)):

			pLoopCity = PyCity( CyGame().getActivePlayer(), cityList[i].getID() )

			bFoodProductionChange = self.isFoodProductionChange (pLoopCity)

			# For each value that needs updating
			for key, value in self.UPDATE_DICT.items():
				# If the key is in this table and it's updatable with the change type
				if dDict.has_key (key) and (bFoodProductionChange or value):
					# Update it
					szValue = self.ColorProblemCityValues (self.CalculateValue (pLoopCity, key), key)
					screen.setTableTextKey (self.m_szMode, dDict[key], pLoopCity.getName(), 1, szValue, WidgetTypes.WIDGET_GENERAL, -1, -1, self.JUSTIFY_DICT[key], len(cityList))

		return

	def handleMainInterfaceUpdate (self):
		if (self.m_bHandlingSpecialistChange and self.m_szMode == self.SPECIAL_AREA_NAME):
			pCity = self.getCurrentCity()
			if (pCity):		
				self.m_nTimesChanged += 1
				return self.isSpecialistChange(pCity)
		return False

	def isSpecialistChange (self, pCity):
		return self.m_szOldSpecialists != self.CalculateValue(pCity, "SPECIALISTS")

	def update(self, fDelta):
		if (self.handleMainInterfaceUpdate()):
#			SpecDomPrint ("Drawing Screen %d" % self.m_nTimesChanged)
			self.drawScreen (self.m_szMode)
			self.m_bHandlingSpecialistChange = False

		return
